from datetime import datetime

from cpl_core.database import TableABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.user import User


class UserJoinedVoiceChannel(TableABC):
    def __init__(
        self,
        user: User,
        channel_id: int,
        joined_on: datetime,
        leaved_on: datetime = None,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._join_id = id
        self._channel_id = channel_id
        self._user = user
        self._joined_on = joined_on
        self._leaved_on = leaved_on

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._join_id

    @property
    def channel_id(self) -> int:
        return self._channel_id

    @property
    @ServiceProviderABC.inject
    def channel_name(self, bot: DiscordBotServiceABC) -> str:
        channel = bot.get_channel(self.channel_id)
        if channel is None:
            return ""
        return channel.name

    @property
    def user(self) -> User:
        return self._user

    @property
    def time(self) -> float:
        if self._leaved_on is None or self._joined_on is None:
            return 0
        return round((self.leaved_on - self.joined_on).total_seconds() / 3600, 2)

    @property
    def joined_on(self) -> datetime:
        return self._joined_on

    @joined_on.setter
    def joined_on(self, value: datetime):
        self._modified_at = datetime.now()
        self.joined_on = value

    @property
    def leaved_on(self) -> datetime:
        return self._leaved_on

    @leaved_on.setter
    def leaved_on(self, value: datetime):
        self._modified_at = datetime.now()
        self._leaved_on = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedVoiceChannel`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedVoiceChannel`
            WHERE `JoinId` = {id};
        """
        )

    @staticmethod
    def get_select_by_user_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedVoiceChannel`
            WHERE `UserId` = {id};
        """
        )

    @staticmethod
    def get_select_active_by_user_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserJoinedVoiceChannel`
            WHERE `UserId` = {id}
            AND `LeavedOn` IS NULL;
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `UserJoinedVoiceChannel` (
                `UserId`, `DiscordChannelId`, `JoinedOn`, `LeavedOn`
            ) VALUES (
                {self._user.id},
                {self._channel_id},
                '{self._joined_on}',
                {"NULL" if self._leaved_on is None else f"'{self._leaved_on}'"}
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `UserJoinedVoiceChannel`
            SET `LeavedOn` = {"NULL" if self._leaved_on is None else f"'{self._leaved_on}'"}
            WHERE `JoinId` = {self._join_id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `UserJoinedVoiceChannel`
            WHERE `JoinId` = {self._join_id};
        """
        )

    @staticmethod
    def delete_by_user_id_string(id: int) -> str:
        return str(
            f"""
            DELETE FROM `UserJoinedVoiceChannel`
            WHERE `UserId` = {id}
    """
        )
