from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_data.abc.table_with_id_abc import TableWithIdABC
from bot_data.model.technician_config_history import TechnicianConfigHistory
from bot_data.model.technician_id_config_history import TechnicianIdConfigHistory
from bot_data.model.technician_ping_url_config_history import TechnicianPingUrlConfigHistory
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC


class TechnicianConfigQuery(DataQueryWithHistoryABC):
    def __init__(self, db: DatabaseContextABC):
        DataQueryWithHistoryABC.__init__(self, "TechnicianConfig", "CFG_TechnicianHistory", TechnicianConfigHistory, db)

        self.set_field("helpCommandReferenceUrl", lambda config, *_: config.help_command_reference_url)
        self.set_field("waitForRestart", lambda config, *_: config.wait_for_restart)
        self.set_field("waitForShutdown", lambda config, *_: config.wait_for_shutdown)
        self.set_field("cacheMaxMessages", lambda config, *_: config.cache_max_messages)
        self.add_collection(
            "featureFlag",
            lambda config, *_: List(any, [{"key": x, "value": config.feature_flags[x]} for x in config.feature_flags]),
        )
        self.set_field("pingURLs", lambda config, *_: config.ping_urls)
        self.set_field("technicianIds", lambda config, *_: config.technician_ids)

        self.set_field("pingURLHistory", self.resolve_ping_url_history)
        self.set_field("technicianIdHistory", self.resolve_technician_id_history)

    def resolve_ping_url_history(self, entry: TableWithIdABC, *_):
        history = List(TechnicianPingUrlConfigHistory)

        results = self._db.select(
            f"""
                SELECT *
                FROM CFG_TechnicianPingUrlsHistory
                WHERE Id = {entry.id}
                ORDER BY DateTo DESC;
            """
        )

        for result in results:
            history.add(TechnicianPingUrlConfigHistory(*result[1:], result[0]))

        return history

    def resolve_technician_id_history(self, entry: TableWithIdABC, *_):
        history = List(TechnicianIdConfigHistory)

        results = self._db.select(
            f"""
                SELECT *
                FROM CFG_TechnicianIdsHistory
                WHERE Id = {entry.id}
                ORDER BY DateTo DESC;
            """
        )

        for result in results:
            history.add(TechnicianIdConfigHistory(*result[1:], result[0]))

        return history
