import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnMessageABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.achievements.achievement_service import AchievementService


class AchievementOnMessageEvent(OnMessageABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        achievements: AchievementService,
        db: DatabaseContextABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
    ):
        OnMessageABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._achievements = achievements
        self._db = db
        self._servers = servers
        self._users = users

    @EventChecks.check_is_ready()
    async def on_message(self, message: discord.Message):
        if message.guild is None:
            return

        if message.author.bot:
            return

        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{message.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(
            server_config.feature_flags, FeatureFlagsEnum.achievements_module
        ):
            return

        server = self._servers.get_server_by_discord_id(message.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(message.author.id, server.id)

        user.message_count += 1
        self._users.update_user(user)
        self._db.save_changes()

        await self._achievements.validate_achievements_for_user(user)
