from typing import Optional

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.base.model.active_event import ActiveEvent


class EventService:
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        db: DatabaseContextABC,
    ):
        self._config = config
        self._logger = logger
        self._servers = servers
        self._users = users
        self._db = db

        self._active_events = List(ActiveEvent)

    def add_event(self, event: ActiveEvent):
        if self._active_events.contains(event):
            return

        self._active_events.add(event)

    def get_active_event(self, event: discord.ScheduledEvent) -> Optional[ActiveEvent]:
        return self._active_events.where(lambda x: x.event.id == event.id).single_or_default()

    def get_active_event_by_channel_id(self, channel_id: int) -> Optional[ActiveEvent]:
        return self._active_events.where(
            lambda x: x.event.channel is not None and x.event.channel.id == channel_id
        ).single_or_default()

    def remove_event(self, event: ActiveEvent):
        if not self._active_events.contains(event):
            return

        self._active_events.remove(event)

    def give_xp_for_event_participation(self, member: discord.Member, active_event: ActiveEvent):
        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        if active_event.participants.any(lambda x: x.id == user.id):
            self._logger.debug(__name__, f"Module {type(self)} stopped")
            return

        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{server.discord_id}")
        user.xp += settings.xp_per_event_participation
        self._users.update_user(user)
        self._db.save_changes()
        active_event.participants.append(user)
